use anyhow::Result;
use clap::{Parser, Subcommand};
use pique::{
    db::{NewUser, UserQuery},
    prelude::*,
};
use rand::distributions::Alphanumeric;
use rand::{distributions::DistString, thread_rng};
use sea_orm::Database;

#[tokio::main]
pub async fn main() -> Result<()> {
    dotenvy::dotenv()?;

    match AdminCli::parse().command {
        AdminCommand::CreateUser {
            full_name,
            email,
            username,
            password,
        } => {
            let password = match password {
                Some(p) => p,
                None => {
                    let mut rng = thread_rng();
                    let password = Alphanumeric.sample_string(&mut rng, 24);

                    println!("Generated password: {}", password);
                    password
                }
            };
            handle_create_user(NewUser {
                full_name,
                email,
                username,
                password,
            })
            .await?
        }
        AdminCommand::ListUsers => handle_list_users().await?,
    };

    Ok(())
}

#[derive(Parser, Debug)]
struct AdminCli {
    #[clap(subcommand)]
    pub command: AdminCommand,
}

#[derive(Subcommand, Debug)]
pub enum AdminCommand {
    CreateUser {
        full_name: String,
        email: String,
        username: String,
        password: Option<String>,
    },

    ListUsers,
}

async fn handle_create_user(new_user: NewUser) -> Result<()> {
    let db = connect_to_db().await?;

    let user = UserQuery(&db).insert(new_user).await?;
    println!("User created successfully with id = {}", user.id.unwrap());

    Ok(())
}

async fn handle_list_users() -> Result<()> {
    let db = connect_to_db().await?;

    let users = UserQuery(&db).all().await?;

    println!("Found {} users.", users.len());
    for user in users {
        println!(" > {}: {} ({})", user.id, user.username, user.full_name);
    }

    Ok(())
}

async fn connect_to_db() -> Result<DatabaseConnection> {
    let db_url = dotenvy::var("DATABASE_URL")?;
    let db = Database::connect(db_url).await?;
    Ok(db)
}
