use sea_orm::Set;
use thiserror::Error;

use crate::{entity::user, password, prelude::*};

pub struct NewUser {
    pub full_name: String,
    pub email: String,
    pub username: String,
    pub password: String,
}

impl NewUser {
    pub fn validate(&self) -> Result<(), Vec<ValidationError>> {
        let mut validation_errors = vec![];

        if self.full_name.len() > 100 {
            validation_errors.push(ValidationError::on("full_name", "too long (max=100)"));
        }

        if self.email.len() > 100 {
            validation_errors.push(ValidationError::on("email", "too long (max=100)"));
        }

        if self.username.len() > 100 {
            validation_errors.push(ValidationError::on("username", "too long (max=32)"));
        }

        if validation_errors.is_empty() {
            Ok(())
        } else {
            Err(validation_errors)
        }
    }

    pub fn hash_password(&self) -> String {
        password::hash(&self.password)
    }
}

#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
pub struct ValidationError {
    pub field: String,
    pub message: String,
}

impl ValidationError {
    pub fn on(field: &str, message: &str) -> ValidationError {
        ValidationError {
            field: field.to_owned(),
            message: message.to_owned(),
        }
    }
}

#[derive(Debug, Error)]
pub enum DbError {
    #[error("internal database error")]
    Internal(#[from] sea_orm::DbErr),
}

pub struct UserQuery<'a>(pub &'a DatabaseConnection);

impl UserQuery<'_> {
    pub async fn insert(&self, new_user: NewUser) -> Result<user::ActiveModel, DbError> {
        let password_hash = new_user.hash_password();
        let user = user::ActiveModel {
            full_name: Set(new_user.full_name),
            email: Set(new_user.email),
            username: Set(new_user.username),
            password_hash: Set(password_hash),
            ..Default::default()
        }
        .save(self.0)
        .await?;
        Ok(user)
    }

    pub async fn all(&self) -> Result<Vec<user::Model>, DbError> {
        let users = User::find().all(self.0).await?;
        Ok(users)
    }

    pub async fn by_id(&self, id: i32) -> Result<Option<user::Model>, DbError> {
        let user = User::find_by_id(id).one(self.0).await?;
        Ok(user)
    }

    pub async fn by_username(&self, username: &str) -> Result<Option<user::Model>, DbError> {
        let user = User::find()
            .filter(user::Column::Username.eq(username))
            .one(self.0)
            .await?;
        Ok(user)
    }
}
