use axum::{response::Redirect, Form};
use axum_login::AuthSession;

use crate::{handler::internal_server_error, prelude::*, session::Credentials};

pub struct LoginTemplate {
    pub username: String,
    pub password: String,
    pub error: Option<String>,
}

pub async fn login_page(
    State(ctx): State<Context>,
    auth_session: AuthSession<Context>,
) -> Response {
    if auth_session.user.is_some() {
        return Redirect::to("/").into_response();
    }

    render_login_page(&ctx, "", "", None)
}

fn render_login_page(
    ctx: &Context,
    username: &str,
    password: &str,
    error: Option<&'static str>,
) -> Response {
    ctx.render_resp(
        "login.html",
        context! {
            username => username,
            password => password,
            error => error,
        },
    )
}

const LOGIN_ERROR_MSG: &str = "Invalid username or password";

pub async fn login_submit(
    State(ctx): State<Context>,
    mut auth_session: AuthSession<Context>,
    Form(creds): Form<Credentials>,
) -> Response {
    match auth_session.authenticate(creds).await {
        Ok(Some(user)) => {
            if let Err(err) = auth_session.login(&user).await {
                error!(?err, "error while logging in user");
                return internal_server_error();
            }

            Redirect::to("/").into_response()
        }
        Ok(None) => render_login_page(&ctx, "", "", Some(LOGIN_ERROR_MSG)),
        Err(err) => {
            error!(?err, "error while authenticating user");
            internal_server_error()
        }
    }
}

pub async fn logout(mut auth_session: AuthSession<Context>) -> Response {
    if let Err(err) = auth_session.logout().await {
        error!(?err, "error while logging out user");
    }

    Redirect::to("/login").into_response()
}

//const INVALID_LOGIN_MESSAGE: &str = "Invalid username/password, please try again.";
//
//pub async fn login_submission(
//    request: HttpRequest,
//    context: web::Data<Context>,
//    form: web::Form<LoginForm>,
//) -> impl Responder {
//    let mut conn = match context.pool.get() {
//        Ok(conn) => conn,
//        Err(_) => return internal_server_error(),
//    };
//
//    let user = match fetch_user_by_username(&mut conn, &form.username) {
//        Ok(Some(user)) => user,
//        Ok(None) => {
//            return LoginTemplate {
//                username: form.username.clone(),
//                password: String::new(),
//                error: Some(INVALID_LOGIN_MESSAGE.into()),
//            }
//            .to_response()
//        }
//        Err(_) => return internal_server_error(),
//    };
//
//    if !user.check_password(&form.password) {
//        return LoginTemplate {
//            username: form.username.clone(),
//            password: String::new(),
//            error: Some(INVALID_LOGIN_MESSAGE.into()),
//        }
//        .to_response();
//    }
//
//    if Identity::login(&request.extensions(), user.id.to_string()).is_err() {
//        return internal_server_error();
//    }
//
//    return HttpResponse::Found()
//        .append_header(("Location", "/"))
//        .finish();
//}
//
//#[get("/logout")]
//pub async fn logout(user: Option<Identity>) -> impl Responder {
//    if let Some(user) = user {
//        user.logout();
//    }
//
//    redirect_to_login()
//}
